print("import: 'atom'")
import atom

print("import: 'gdata'")
import gdata

print("import: 'gdata.Crypto'")
import gdata.Crypto

print("import: 'gdata.Crypto.Cipher'")
import gdata.Crypto.Cipher

print("import: 'gdata.Crypto.Hash'")
import gdata.Crypto.Hash

print("import: 'gdata.Crypto.Protocol'")
import gdata.Crypto.Protocol

print("import: 'gdata.Crypto.PublicKey'")
import gdata.Crypto.PublicKey

print("import: 'gdata.Crypto.Util'")
import gdata.Crypto.Util

print("import: 'gdata.acl'")
import gdata.acl

print("import: 'gdata.alt'")
import gdata.alt

print("import: 'gdata.analytics'")
import gdata.analytics

print("import: 'gdata.apps'")
import gdata.apps

print("import: 'gdata.apps.adminsettings'")
import gdata.apps.adminsettings

print("import: 'gdata.apps.audit'")
import gdata.apps.audit

print("import: 'gdata.apps.emailsettings'")
import gdata.apps.emailsettings

print("import: 'gdata.apps.groups'")
import gdata.apps.groups

print("import: 'gdata.apps.migration'")
import gdata.apps.migration

print("import: 'gdata.apps.multidomain'")
import gdata.apps.multidomain

print("import: 'gdata.apps.organization'")
import gdata.apps.organization

print("import: 'gdata.blogger'")
import gdata.blogger

print("import: 'gdata.books'")
import gdata.books

print("import: 'gdata.calendar'")
import gdata.calendar

print("import: 'gdata.calendar_resource'")
import gdata.calendar_resource

print("import: 'gdata.codesearch'")
import gdata.codesearch

print("import: 'gdata.contacts'")
import gdata.contacts

print("import: 'gdata.contentforshopping'")
import gdata.contentforshopping

print("import: 'gdata.docs'")
import gdata.docs

print("import: 'gdata.dublincore'")
import gdata.dublincore

print("import: 'gdata.exif'")
import gdata.exif

print("import: 'gdata.finance'")
import gdata.finance

print("import: 'gdata.geo'")
import gdata.geo

print("import: 'gdata.health'")
import gdata.health

print("import: 'gdata.media'")
import gdata.media

print("import: 'gdata.notebook'")
import gdata.notebook

print("import: 'gdata.oauth'")
import gdata.oauth

print("import: 'gdata.opensearch'")
import gdata.opensearch

print("import: 'gdata.photos'")
import gdata.photos

print("import: 'gdata.projecthosting'")
import gdata.projecthosting

print("import: 'gdata.sites'")
import gdata.sites

print("import: 'gdata.spreadsheet'")
import gdata.spreadsheet

print("import: 'gdata.spreadsheets'")
import gdata.spreadsheets

print("import: 'gdata.tlslite'")
import gdata.tlslite

print("import: 'gdata.tlslite.integration'")
import gdata.tlslite.integration

print("import: 'gdata.tlslite.utils'")
import gdata.tlslite.utils

print("import: 'gdata.webmastertools'")
import gdata.webmastertools

print("import: 'gdata.youtube'")
import gdata.youtube

